@@ -18,6 +18,7 @@ var replaceStars = semver.replaceStars;
 var toComparators = semver.toComparators;
 var SemVer = semver.SemVer;
 var Range = semver.Range;
+var Comparator = semver.Comparator;
 
 test('\ncomparison tests', function(t) {
   // [version1, version2]
@@ -188,6 +189,8 @@ test('\nrange tests', function(t) {
     ['*', '1.2.3'],
     ['2', '2.1.2'],
     ['2.3', '2.3.1'],
+    ['~x', '0.0.9'], // >=2.4.0 <2.5.0
+    ['~2', '2.0.9'], // >=2.4.0 <2.5.0
     ['~2.4', '2.4.0'], // >=2.4.0 <2.5.0
     ['~2.4', '2.4.5'],
     ['~>3.2.1', '3.2.2'], // >=3.2.1 <3.3.0,
@@ -221,11 +224,19 @@ test('\nrange tests', function(t) {
     ['^1.2.3', '1.8.1'],
     ['^0.1.2', '0.1.2'],
     ['^0.1', '0.1.2'],
+    ['^0.0.1', '0.0.1'],
     ['^1.2', '1.4.2'],
     ['^1.2 ^1', '1.4.2'],
     ['^1.2.3-alpha', '1.2.3-pre'],
     ['^1.2.0-alpha', '1.2.0-pre'],
-    ['^0.0.1-alpha', '0.0.1-beta']
+    ['^0.0.1-alpha', '0.0.1-beta'],
+    ['^0.1.1-alpha', '0.1.1-beta'],
+    ['^x', '1.2.3'],
+    ['x - 1.0.0', '0.9.7'],
+    ['x - 1.x', '0.9.7'],
+    ['1.0.0 - x', '1.9.7'],
+    ['1.x - x', '1.9.7'],
+    ['<=7.x', '7.9.9']
   ].forEach(function(v) {
     var range = v[0];
     var ver = v[1];
@@ -298,6 +309,7 @@ test('\nnegative range tests', function(t) {
     ['<1.2.3', '1.2.3-beta'],
     ['=1.2.3', '1.2.3-beta'],
     ['>1.2', '1.2.8'],
+    ['^0.0.1', '0.0.2'],
     ['^1.2.3', '2.0.0-alpha'],
     ['^1.2.3', '1.2.2'],
     ['^1.2', '1.1.9'],
@@ -305,7 +317,8 @@ test('\nnegative range tests', function(t) {
     // invalid ranges never satisfied!
     ['blerg', '1.2.3'],
     ['git+https://user:password0123@github.com/foo', '123.0.0', true],
-    ['^1.2.3', '2.0.0-pre']
+    ['^1.2.3', '2.0.0-pre'],
+    ['^1.2.3', false]
   ].forEach(function(v) {
     var range = v[0];
     var ver = v[1];
@@ -663,6 +676,7 @@ test('\nstrict vs loose version numbers', function(t) {
     t.throws(function() {
       new SemVer(strict).compare(loose);
     });
+    t.equal(semver.compareLoose(v[0], v[1]), 0);
   });
   t.end();
 });
@@ -712,3 +726,174 @@ test('\nmin satisfying', function(t) {
   });
   t.end();
 });
+
+test('\nintersect comparators', function(t) {
+  [
+    // One is a Version
+    ['1.3.0', '>=1.3.0', true],
+    ['1.3.0', '>1.3.0', false],
+    ['>=1.3.0', '1.3.0', true],
+    ['>1.3.0', '1.3.0', false],
+    // Same direction increasing
+    ['>1.3.0', '>1.2.0', true],
+    ['>1.2.0', '>1.3.0', true],
+    ['>=1.2.0', '>1.3.0', true],
+    ['>1.2.0', '>=1.3.0', true],
+    // Same direction decreasing
+    ['<1.3.0', '<1.2.0', true],
+    ['<1.2.0', '<1.3.0', true],
+    ['<=1.2.0', '<1.3.0', true],
+    ['<1.2.0', '<=1.3.0', true],
+    // Different directions, same semver and inclusive operator
+    ['>=1.3.0', '<=1.3.0', true],
+    ['>=v1.3.0', '<=1.3.0', true],
+    ['>=1.3.0', '>=1.3.0', true],
+    ['<=1.3.0', '<=1.3.0', true],
+    ['<=1.3.0', '<=v1.3.0', true],
+    ['>1.3.0', '<=1.3.0', false],
+    ['>=1.3.0', '<1.3.0', false],
+    // Opposite matching directions
+    ['>1.0.0', '<2.0.0', true],
+    ['>=1.0.0', '<2.0.0', true],
+    ['>=1.0.0', '<=2.0.0', true],
+    ['>1.0.0', '<=2.0.0', true],
+    ['<=2.0.0', '>1.0.0', true],
+    ['<=1.0.0', '>=2.0.0', false]
+  ].forEach(function(v) {
+    var comparator1 = new Comparator(v[0]);
+    var comparator2 = new Comparator(v[1]);
+    var expect = v[2];
+
+    var actual1 = comparator1.intersects(comparator2);
+    var actual2 = comparator2.intersects(comparator1);
+    var actual3 = semver.intersects(comparator1, comparator2);
+    var actual4 = semver.intersects(comparator2, comparator1);
+    var actual4 = semver.intersects(comparator1, comparator2, true);
+    var actual5 = semver.intersects(comparator2, comparator1, true);
+    var actual6 = semver.intersects(v[0], v[1]);
+    var actual7 = semver.intersects(v[1], v[0]);
+    var actual8 = semver.intersects(v[0], v[1], true);
+    var actual9 = semver.intersects(v[1], v[0], true);
+    t.equal(actual1, expect);
+    t.equal(actual2, expect);
+    t.equal(actual3, expect);
+    t.equal(actual4, expect);
+    t.equal(actual5, expect);
+    t.equal(actual6, expect);
+    t.equal(actual7, expect);
+    t.equal(actual8, expect);
+    t.equal(actual9, expect);
+  });
+  t.end();
+});
+
+test('\nmissing comparator parameter in intersect comparators', function(t) {
+  t.throws(function() {
+      new Comparator('>1.0.0').intersects();
+    }, new TypeError('a Comparator is required'),
+    'throws type error');
+  t.end();
+});
+
+test('\nranges intersect', function(t) {
+  [
+    ['1.3.0 || <1.0.0 >2.0.0', '1.3.0 || <1.0.0 >2.0.0', true],
+    ['<1.0.0 >2.0.0', '>0.0.0', true],
+    ['<1.0.0 >2.0.0', '>1.4.0 <1.6.0', false],
+    ['<1.0.0 >2.0.0', '>1.4.0 <1.6.0 || 2.0.0', false],
+    ['>1.0.0 <=2.0.0', '2.0.0', true],
+    ['<1.0.0 >=2.0.0', '2.1.0', false],
+    ['<1.0.0 >=2.0.0', '>1.4.0 <1.6.0 || 2.0.0', false]
+  ].forEach(function(v) {
+    var range1 = new Range(v[0]);
+    var range2 = new Range(v[1]);
+    var expect = v[2];
+    var actual1 = range1.intersects(range2);
+    var actual2 = range2.intersects(range1);
+    var actual3 = semver.intersects(v[1], v[0]);
+    var actual4 = semver.intersects(v[0], v[1]);
+    var actual5 = semver.intersects(v[1], v[0], true);
+    var actual6 = semver.intersects(v[0], v[1], true);
+    var actual7 = semver.intersects(range1, range2);
+    var actual8 = semver.intersects(range2, range1);
+    var actual9 = semver.intersects(range1, range2, true);
+    var actual0 = semver.intersects(range2, range1, true);
+    t.equal(actual1, expect);
+    t.equal(actual2, expect);
+    t.equal(actual3, expect);
+    t.equal(actual4, expect);
+    t.equal(actual5, expect);
+    t.equal(actual6, expect);
+    t.equal(actual7, expect);
+    t.equal(actual8, expect);
+    t.equal(actual9, expect);
+    t.equal(actual0, expect);
+  });
+  t.end();
+});
+
+test('\nmissing range parameter in range intersect', function(t) {
+  t.throws(function() {
+      new Range('1.0.0').intersects();
+    }, new TypeError('a Range is required'),
+    'throws type error');
+  t.end();
+});
+
+test('outside with bad hilo throws', function(t) {
+  t.throws(function() {
+    semver.outside('1.2.3', '>1.5.0', 'blerg', true)
+  }, new TypeError('Must provide a hilo val of "<" or ">"'));
+  t.end()
+})
+
+test('comparator testing', function(t) {
+  var c = new Comparator('>=1.2.3');
+  t.ok(c.test('1.2.4'));
+  var c2 = new Comparator(c);
+  t.ok(c2.test('1.2.4'));
+  var c3 = new Comparator(c, true);
+  t.ok(c3.test('1.2.4'));
+  t.end()
+});
+
+test('tostrings', function(t) {
+  t.equal(Range('>= v1.2.3').toString(), '>=1.2.3')
+  t.equal(Comparator('>= v1.2.3').toString(), '>=1.2.3')
+  t.end()
+})
+
+test('invalid cmp usage', function(t) {
+  t.throws(function() {
+    cmp('1.2.3', 'a frog', '4.5.6');
+  }, new TypeError('Invalid operator: a frog'));
+  t.end();
+});
+
+test('sorting', function(t) {
+  var list = [
+    '1.2.3',
+    '5.9.6',
+    '0.1.2'
+  ];
+  var sorted = [
+    '0.1.2',
+    '1.2.3',
+    '5.9.6'
+  ];
+  var rsorted = [
+    '5.9.6',
+    '1.2.3',
+    '0.1.2'
+  ];
+  t.same(semver.sort(list), sorted);
+  t.same(semver.rsort(list), rsorted);
+  t.end();
+});
+
+test('bad ranges in max/min satisfying', function(t) {
+  var r = 'some frogs and sneks-v2.5.6';
+  t.equal(semver.maxSatisfying([], r), null);
+  t.equal(semver.minSatisfying([], r), null);
+  t.end();
+});
